﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.ServiceModel;
using HeuristicLab.Services.Optimization.ControllerService;
using HeuristicLab.Services.Optimization.ControllerService.Model;
using System.ServiceModel.Description;
using System.Web.Security;
using HeuristicLab.Services.Optimization.Web.Models;
using HeuristicLab.Services.Optimization.ControllerService.General;

namespace HeuristicLab.Services.Optimization.Web.Controllers
{
    [Authorize(Roles="Web User")]
    public class OptimizationController : Controller
    {
      public static readonly string DEFAULT_CONTROLLER_ENDPOINT = Configuration.ControllerEndpointName;

      public delegate T ControllerServiceDelegate<T>(IControllerService service);
      public delegate void ControllerServiceDelegate(IControllerService service);

      public T withControllerService<T>(ControllerServiceDelegate<T> del) {
        if (Session["pw"] == null)
          Response.Redirect("/Account/Logon");

        using (var cf = new ChannelFactory<IControllerService>(DEFAULT_CONTROLLER_ENDPOINT)) {
          var credentialBehaviour = cf.Endpoint.Behaviors.Find<ClientCredentials>();
          credentialBehaviour.UserName.UserName = Membership.GetUser().UserName;
          credentialBehaviour.UserName.Password = Session["pw"] as string;
          var controllerProxy = cf.CreateChannel();
          return del(controllerProxy);
        }
      }

      public void withControllerService(ControllerServiceDelegate del) {
        if (Session["pw"] == null)
          Response.Redirect("/Account/Logon");

        using (var cf = new ChannelFactory<IControllerService>(DEFAULT_CONTROLLER_ENDPOINT)) {          
          var credentialBehaviour = cf.Endpoint.Behaviors.Find<ClientCredentials>();
          credentialBehaviour.UserName.UserName = Membership.GetUser().UserName;
          credentialBehaviour.UserName.Password = Session["pw"] as string;          
          var controllerProxy = cf.CreateChannel();
          del(controllerProxy);
        }
      }
        //
        // GET: /Optimization/        

        public ActionResult Index()
        {
          var optModel = withControllerService<OptimizationModel>((service) => {
            OptimizationModel model = new OptimizationModel();
            model.Scenarios = service.GetOptimizationScenarios();
            User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
            model.Jobs = service.GetJobs(u);
            return model;
          });
          return View(optModel);
        }

        public ActionResult ProblemParameters(string scenario) {
          var optScenario = withControllerService<OptimizationScenario>((service) => {
            return service.GetOptimizationScenarioByName(scenario);
          });
          Session["scenario"] = optScenario;
          if (optScenario.FirstAlgorithm.Problem == null)
            return RedirectToAction("AlgorithmParameters");
          return View(optScenario.FirstAlgorithm.Problem.Parameters);
        }

        public ActionResult ProblemParameters2() {                    
          return View("ProblemParameters", (Session["scenario"] as OptimizationScenario).FirstAlgorithm.Problem.Parameters);
        }

        public ActionResult JobDetails(string jobId) {
          Job job = withControllerService<Job>((service) => {
            User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
            return service.GetJob(u, jobId);
          });
          var runs = withControllerService<IList<Run>>((service) => {
            User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
            return service.GetJobResults(u, jobId);
          });
          JobDetailsModel jdm = new JobDetailsModel() { Job = job, Runs = runs };
          Session["jobDetails"] = jdm;
          return View(jdm);
        }

        [HttpPost]
        public ActionResult JobDetails(Job job) {
          withControllerService((service) => {
            User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
            service.DeleteJob(u, job.Id);
          });
          return RedirectToAction("Index");
        }

        [HttpPost]
        public ActionResult ProblemParameters(ProblemParameters parameters) {
          if (ModelState.IsValid) {
            return RedirectToAction("AlgorithmParameters", parameters);
          } 
          return View(parameters);        
        }

        public ActionResult AlgorithmParameters() {
          return View((Session["scenario"] as OptimizationScenario).FirstAlgorithm.Parameters);
        }

        [HttpPost]
        public ActionResult AlgorithmParameters(AlgorithmParameters parameters) {
          if (ModelState.IsValid) {
            return RedirectToAction("ScheduleJob");
          }
          return View(parameters);
        }

        public ActionResult ScheduleJob() {
          return View(Session["scenario"]);
        }

        [HttpPost]
        public ActionResult ScheduleJob(ScheduleJobModel jobModel) {
          using (var cf = new ChannelFactory<IControllerService>(OptimizationController.DEFAULT_CONTROLLER_ENDPOINT)) {
            var credentialBehaviour = cf.Endpoint.Behaviors.Find<ClientCredentials>();
            credentialBehaviour.UserName.UserName = Membership.GetUser().UserName;
            credentialBehaviour.UserName.Password = Session["pw"] as string;
            var controllerProxy = cf.CreateChannel();
            User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
            var scenario = Session["scenario"] as OptimizationScenario;
            var details = new JobExecutionDetails() { Group = jobModel.Group, JobTitle = jobModel.Name, Repititions = jobModel.Repetitions };
            controllerProxy.ScheduleOptimizationScenario(u, scenario, details);
          }
          return RedirectToAction("Index");
        }
        // TODO: Add unique identifier for algorithm / input parameters
        public ActionResult AddRow(int id, string paramName) {
          var os = Session["scenario"] as OptimizationScenario;
          var parameter = os.FirstAlgorithm.Problem.Parameters.FindByName(paramName);
          bool inputParam = true;
          if (parameter == null) {
            parameter = os.FirstAlgorithm.Parameters.FindByName(paramName);
            inputParam = false;
          }
          var dm = parameter.Value as DecimalMatrix;
          dm.CopyRow(id);
          return inputParam ? View("ProblemParameters", os.FirstAlgorithm.Problem.Parameters) : View("AlgorithmParameters", os.FirstAlgorithm.Parameters);
        }

        public ActionResult DeleteRow(int id, string paramName) {
          var os = Session["scenario"] as OptimizationScenario;
          var parameter = os.FirstAlgorithm.Problem.Parameters.FindByName(paramName);
          bool inputParam = true;
          if (parameter == null) {
            parameter = os.FirstAlgorithm.Parameters.FindByName(paramName);
            inputParam = false;
          }
          var dm = parameter.Value as DecimalMatrix;
          dm.DeleteRow(id);
          return inputParam ? View("ProblemParameters", os.FirstAlgorithm.Problem.Parameters) : View("AlgorithmParameters", os.FirstAlgorithm.Parameters);
        }
    }
}
